@@ -19,9 +19,9 @@
  *
  */
 
-
-
-#define SOURCE_VERSION "2011.2.0"
+#ifndef SOURCE_VERSION
+#define SOURCE_VERSION "2011.3.0"
+#endif
 
 #define SOCKET_PATH_FMT "%s/batman_adv/%s/socket"
 
